import codecs
import sqlite3
from functools import partial

import source_code
from event_handler import EventHandler
from source_code_event_handler import SourceCodeEventHandler
from stateful_event_handler import StatefulEventHandler
from tornado.ioloop import IOLoop

from login_component import authorize_login
from util import create_source_code_response_data

login_component_py_response = partial(
    create_source_code_response_data, filename='login_component.py', language='python'
)


def echo_handler(data):
    return data


def rot13_handler(data):
    data['data'] = codecs.encode(data['data'], 'rot13')
    return data


def change_case_handler(data):
    data['data'] = data['data'].upper() if data['data'].islower() else data['data'].lower()
    return data


def reverse_handler(data):
    data['data'] = data['data'][::-1]
    return data


def login_handler(data):
    email, password = data['data']['email'], data['data']['password']
    try:
        sql_statement = source_code.find_local_variable_value(authorize_login, 'sql_statement')
        yield (
            'anchor_logger',
            'The SQL statement executed by the server will look like this:\n `{}`'.format(sql_statement)
        )

        yield ('anchor_webide',
               login_component_py_response(content=source_code.get_source_code(authorize_login, strip_comments=False)))

        sql_statement_with_values = sql_statement.format(email, password)
        yield (
            'anchor_logger',
            'After the submitted parameters are substituted it looks like this:\n `{}`'.format(
                sql_statement_with_values
            )
        )

        logged_in_email, is_admin = authorize_login(email, password)

        yield (
            'anchor_logger',
            'After the query is executed, it returns _{}_ as email address, and _{}_ for is_admin'.format(
                logged_in_email, is_admin
            )
        )

        if logged_in_email is not None:
            response = 'Logged in as _{}_. You __{}have__ admin privileges.'.format(
                logged_in_email,
                '' if is_admin else 'don\'t '
            )
        else:
            response = 'Bad username/password!'
    except sqlite3.Warning:
        response = 'Invalid request!'

    yield ('anchor_login', '# Login page\n' + response)


if __name__ == '__main__':
    anchor_a = EventHandler('anchor_a', change_case_handler)
    anchor_b = EventHandler('anchor_b', rot13_handler)
    anchor_c = EventHandler('anchor_c', reverse_handler)
    anchor_login = StatefulEventHandler('anchor_login', login_handler)
    anchor_webide = SourceCodeEventHandler('anchor_webide', 'login_component.py', 'login')
    IOLoop.instance().start()

